==> kameleoon_client_flutter-1.0.2.tar.gz: lib/src/internal/kameleoon_client_impl_web.dart <==
@JS()
library kameleoon_client.js;

import 'package:js/js.dart';
import 'package:js/js_util.dart';


import '../kameleoon_client.dart';
import '../kameleoon_data.dart' as KameleoonData;
import '../kameleoon_exception.dart';

/// Implementation of [KameleoonClient] for Web platform
class KameleoonClientImpl extends KameleoonClient {
  late final _KameleoonClientJs _kameleoonClientJs;

  bool ready = false;

  KameleoonClientImpl(String siteCode)
      : _kameleoonClientJs = _KameleoonClientJs(siteCode);

  Future<void> trackConversion(String visitorCode, int goalID,
      [double? revenue]) async {
    try  {
      if (revenue != null) {
        _kameleoonClientJs.trackConversion(visitorCode, goalID, revenue);
      } else {
        _kameleoonClientJs.trackConversion(visitorCode, goalID);
      }
    } catch (e) {
      final exception = _jsToMap(e);
      switch (exception["type"]) {
        case "VisitorCodeNotValid":
          throw VisitorCodeNotValid(exception["message"]);
        default:
          throw Exception(e);
      }
    }
  }

  Future<int?> triggerExperiment(String visitorCode, int experimentID) async {
    try {
      return await _kameleoonClientJs.triggerExperiment(
          visitorCode, experimentID);
    } catch (e) {
      final exception = _jsToMap(e);
      switch (exception["type"]) {
        case "VisitorCodeNotValid":
          throw VisitorCodeNotValid(exception["message"]);
        case "ExperimentConfigurationNotFound":
          throw ExperimentConfigurationNotFound();
        case "NotTargeted":
          throw NotTargeted();
        case "NotActivated(":
          throw NotActivated();
        default:
          throw Exception(e);
      }
    }
  }

  Future<bool?> activateFeature(String visitorCode, var featureKeyOrID) async {
    try {
      return await _kameleoonClientJs.activateFeature(
          visitorCode, featureKeyOrID);
    } catch (e) {
      final exception = _jsToMap(e);
      switch (exception["type"]) {
        case "VisitorCodeNotValid":
          throw VisitorCodeNotValid(exception["message"]);
        case "FeatureConfigurationNotFound":
          throw FeatureConfigurationNotFound();
        case "NotTargeted":
          throw NotTargeted();
        default:
          throw Exception(e);
      }
    }
  }

  Future<String?> obtainVisitorCode([String? defaultVisitorCode]) async =>
      await _kameleoonClientJs.obtainVisitorCode();

  Future<dynamic> obtainVariationAssociatedData(int variationID) async {
    try {
      return _jsToMap(
          await _kameleoonClientJs.obtainVariationAssociatedData(variationID));
    } catch (e) {
      switch (_jsToMap(e)["type"]) {
        case "VariationConfigurationNotFound":
          throw VariationConfigurationNotFound();
        default:
          throw Exception(e);
      }
    }
  }

  Future<Object?> obtainFeatureVariable(
      var featureKeyOrID, String variableKey) async {
    try {
      final variable = await _kameleoonClientJs.obtainFeatureVariable(
          featureKeyOrID, variableKey);
      return variable != null ? variable : throw FeatureConfigurationNotFound();
    } catch (e) {
      if (e is Exception) {
        throw e;
      } else {
        switch (_jsToMap(e)["type"]) {
          case "FeatureConfigurationNotFound":
            throw FeatureConfigurationNotFound();
          default:
            throw Exception(e);
        }
      }
    }
  }

  bool? isReady() {
    return this.ready;
  }

  Future<void> addData(String visitorCode, KameleoonData.Data data) async {
    late final DataJs dataJs;
    try {
      switch (data.dataType) {
        case "CustomData":
          final customData = data as KameleoonData.CustomData;
          dataJs = _CustomDataJs(customData.id!, customData.value!);
          break;
        case "PageView":
          final pageView = data as KameleoonData.PageView;
          dataJs =
              _PageViewJs(pageView.url!, pageView.title!, pageView.referrer!);
          break;
        case "Interest":
          final interest = data as KameleoonData.Interest;
          dataJs = _InterestJs(interest.index!);
          break;
        case "Conversion":
          final conversion = data as KameleoonData.Conversion;
          dataJs = _ConversionJs(
              conversion.goalId!, conversion.revenue, conversion.negative);
          break;
        case "Browser":
          final browser = data as KameleoonData.Browser;
          dataJs = _BrowserJs(browser.browser!);
          break;
      }
      await _kameleoonClientJs.addData(visitorCode, dataJs);
    } catch (e) {
      final exception = _jsToMap(e);
      switch (exception["type"]) {
        case "VisitorCodeNotValid":
          throw VisitorCodeNotValid(exception["message"]);
        default:
            throw Exception(e);
        }        
    }
  }

  void runWhenReady(Function readyCallback, Function failCallback,
      [int timeout = 2000]) async {
    if (this.ready) {
      await readyCallback.call();
    } else {
      _kameleoonClientJs.runWhenReady(allowInterop(() {
        this.ready = true;
        readyCallback.call();
      }), allowInterop(failCallback), timeout);
    }
  }

  Future<void> flush([String? visitorCode]) async {
    try  {
      if (visitorCode != null) {
        _kameleoonClientJs.flush(visitorCode);
      } else {
        _kameleoonClientJs.flush();
      }
    } catch (e) {
      final exception = _jsToMap(e);
      switch (exception["type"]) {
        case "VisitorCodeNotValid":
          throw VisitorCodeNotValid(exception["message"]);
        default:
          throw Exception(e);
      }
    }
  }
}

/// Helper class KameleoonClientJs to make interaction with JS sdk (web platform)
@JS("KameleoonClient")
class _KameleoonClientJs {
  external _KameleoonClientJs(String siteCode);
  external obtainVisitorCode();
  external runWhenReady(Function readyCallback, Function failCallback,
      [int timeout]);
  external obtainFeatureVariable(var featureKeyOrId, String variableKey);
  external obtainVariationAssociatedData(int variationID);
  external flush([String? visitorCode]);
  external activateFeature(String visitorCode, var featureKeyOrId);
  external triggerExperiment(String visitorCode, int experimentID);
  external trackConversion(String visitorCode, int goalId, [double? revenue]);
  external addData(String visitorCode, DataJs data);
}

@JS("Data")
class DataJs {
  external DataJs();
}

@JS("CustomData")
class _CustomDataJs extends DataJs {
  external _CustomDataJs(int id, String value);
}

@JS("Browser")
class _BrowserJs extends DataJs {
  external _BrowserJs(int id);
}

@JS("Interest")
class _InterestJs extends DataJs {
  external _InterestJs(int index);
}

@JS("PageView")
class _PageViewJs extends DataJs {
  external _PageViewJs(String url, String title, int referrer);
}

@JS("Conversion")
class _ConversionJs extends DataJs {
  external _ConversionJs(int goalId, [double? revenue, bool? negative]);
}

Map _jsToMap(jsObject) {
  return new Map.fromIterable(
    _getKeysOfObject(jsObject),
    value: (key) => getProperty(jsObject, key),
  );
}

@JS('Object.keys')
external List<String> _getKeysOfObject(jsObject);

==> kameleoon_client_flutter-1.0.3.tar.gz: lib/src/internal/kameleoon_client_impl_web.dart <==
@JS()
library kameleoon_client.js;

import 'package:js/js.dart';
import 'package:js/js_util.dart';


import '../kameleoon_client.dart';
import '../kameleoon_data.dart' as KameleoonData;
import '../kameleoon_exception.dart';

/// Implementation of [KameleoonClient] for Web platform
class KameleoonClientImpl extends KameleoonClient {
  late final _KameleoonClientJs _kameleoonClientJs;

  bool ready = false;

  KameleoonClientImpl(String siteCode, [String environment = "production"]) {
    _kameleoonClientJs = _KameleoonClientJs(siteCode, _SDKConfigurationJs(environment: environment));
  }

  Future<void> trackConversion(String visitorCode, int goalID,
      [double? revenue]) async {
    try  {
      if (revenue != null) {
        _kameleoonClientJs.trackConversion(visitorCode, goalID, revenue);
      } else {
        _kameleoonClientJs.trackConversion(visitorCode, goalID);
      }
    } catch (e) {
      final exception = _jsToMap(e);
      switch (exception["type"]) {
        case "VisitorCodeNotValid":
          throw VisitorCodeNotValid(exception["message"]);
        default:
          throw Exception(e);
      }
    }
  }

  Future<int?> triggerExperiment(String visitorCode, int experimentID) async {
    try {
      return await _kameleoonClientJs.triggerExperiment(
          visitorCode, experimentID);
    } catch (e) {
      final exception = _jsToMap(e);
      switch (exception["type"]) {
        case "VisitorCodeNotValid":
          throw VisitorCodeNotValid(exception["message"]);
        case "ExperimentConfigurationNotFound":
          throw ExperimentConfigurationNotFound();
        case "NotTargeted":
          throw NotTargeted();
        case "NotActivated(":
          throw NotActivated();
        default:
          throw Exception(e);
      }
    }
  }

  Future<bool?> activateFeature(String visitorCode, var featureKeyOrID) async {
    try {
      return await _kameleoonClientJs.activateFeature(
          visitorCode, featureKeyOrID);
    } catch (e) {
      final exception = _jsToMap(e);
      switch (exception["type"]) {
        case "VisitorCodeNotValid":
          throw VisitorCodeNotValid(exception["message"]);
        case "FeatureConfigurationNotFound":
          throw FeatureConfigurationNotFound();
        case "NotTargeted":
          throw NotTargeted();
        default:
          throw Exception(e);
      }
    }
  }

  Future<String?> obtainVisitorCode([String? defaultVisitorCode]) async =>
      await _kameleoonClientJs.obtainVisitorCode();

  Future<dynamic> obtainVariationAssociatedData(int variationID) async {
    try {
      return _jsToMap(
          await _kameleoonClientJs.obtainVariationAssociatedData(variationID));
    } catch (e) {
      switch (_jsToMap(e)["type"]) {
        case "VariationConfigurationNotFound":
          throw VariationConfigurationNotFound();
        default:
          throw Exception(e);
      }
    }
  }

  Future<Object?> obtainFeatureVariable(
      var featureKeyOrID, String variableKey) async {
    try {
      final variable = await _kameleoonClientJs.obtainFeatureVariable(
          featureKeyOrID, variableKey);
      return variable != null ? variable : throw FeatureConfigurationNotFound();
    } catch (e) {
      if (e is Exception) {
        throw e;
      } else {
        switch (_jsToMap(e)["type"]) {
          case "FeatureConfigurationNotFound":
            throw FeatureConfigurationNotFound();
          default:
            throw Exception(e);
        }
      }
    }
  }

  bool? isReady() {
    return this.ready;
  }

  Future<void> addData(String visitorCode, KameleoonData.Data data) async {
    late final DataJs dataJs;
    try {
      switch (data.dataType) {
        case "CustomData":
          final customData = data as KameleoonData.CustomData;
          dataJs = _CustomDataJs(customData.id!, customData.value!);
          break;
        case "PageView":
          final pageView = data as KameleoonData.PageView;
          dataJs =
              _PageViewJs(pageView.url!, pageView.title!, pageView.referrer!);
          break;
        case "Interest":
          final interest = data as KameleoonData.Interest;
          dataJs = _InterestJs(interest.index!);
          break;
        case "Conversion":
          final conversion = data as KameleoonData.Conversion;
          dataJs = _ConversionJs(
              conversion.goalId!, conversion.revenue, conversion.negative);
          break;
        case "Browser":
          final browser = data as KameleoonData.Browser;
          dataJs = _BrowserJs(browser.browser!);
          break;
      }
      await _kameleoonClientJs.addData(visitorCode, dataJs);
    } catch (e) {
      final exception = _jsToMap(e);
      switch (exception["type"]) {
        case "VisitorCodeNotValid":
          throw VisitorCodeNotValid(exception["message"]);
        default:
            throw Exception(e);
        }        
    }
  }

  void runWhenReady(Function readyCallback, Function failCallback,
      [int timeout = 2000]) async {
    if (this.ready) {
      await readyCallback.call();
    } else {
      _kameleoonClientJs.runWhenReady(allowInterop(() {
        this.ready = true;
        readyCallback.call();
      }), allowInterop(failCallback), timeout);
    }
  }

  Future<void> flush([String? visitorCode]) async {
    try  {
      if (visitorCode != null) {
        _kameleoonClientJs.flush(visitorCode);
      } else {
        _kameleoonClientJs.flush();
      }
    } catch (e) {
      final exception = _jsToMap(e);
      switch (exception["type"]) {
        case "VisitorCodeNotValid":
          throw VisitorCodeNotValid(exception["message"]);
        default:
          throw Exception(e);
      }
    }
  }

  Future<dynamic> retrieveDataFromRemoteSource(String key, [Function? callback]) async {
    return _jsToMap(await promiseToFuture(_kameleoonClientJs.retrieveDataFromRemoteSource(key)));
  }
}

/// Helper class KameleoonClientJs to make interaction with JS sdk (web platform)
@JS("KameleoonClient")
class _KameleoonClientJs {
  external _KameleoonClientJs(String siteCode, _SDKConfigurationJs configuration);
  external obtainVisitorCode();
  external runWhenReady(Function readyCallback, Function failCallback,
      [int timeout]);
  external obtainFeatureVariable(var featureKeyOrId, String variableKey);
  external obtainVariationAssociatedData(int variationID);
  external flush([String? visitorCode]);
  external activateFeature(String visitorCode, var featureKeyOrId);
  external triggerExperiment(String visitorCode, int experimentID);
  external trackConversion(String visitorCode, int goalId, [double? revenue]);
  external addData(String visitorCode, DataJs data);
  external retrieveDataFromRemoteSource(String key);
}

@JS("Data")
class DataJs {
  external DataJs();
}

@JS("CustomData")
class _CustomDataJs extends DataJs {
  external _CustomDataJs(int id, String value);
}

@JS("Browser")
class _BrowserJs extends DataJs {
  external _BrowserJs(int id);
}

@JS("Interest")
class _InterestJs extends DataJs {
  external _InterestJs(int index);
}

@JS("PageView")
class _PageViewJs extends DataJs {
  external _PageViewJs(String url, String title, int referrer);
}

@JS("Conversion")
class _ConversionJs extends DataJs {
  external _ConversionJs(int goalId, [double? revenue, bool? negative]);
}

@JS()
@anonymous
class _SDKConfigurationJs {
  external String environment;
  external factory _SDKConfigurationJs({String environment});
}

Map _jsToMap(jsObject) {
  return new Map.fromIterable(
    _getKeysOfObject(jsObject),
    value: (key) => getProperty(jsObject, key),
  );
}

@JS('Object.keys')
external List<String> _getKeysOfObject(jsObject);
